@@ -4,7 +4,7 @@
  *   Copyright (C) 2001-2011, 2013-2020 Free Software Foundation, Inc.    *
  *   Copyright (C) 2014 Mike Frysinger                                    *
  *   Copyright (C) 2019 Brand Huntsman                                    *
- *   Copyright (C) 2014-2019 Benno Schulenberg                            *
+ *   Copyright (C) 2014-2020 Benno Schulenberg                            *
  *                                                                        *
  *   GNU nano is free software: you can redistribute it and/or modify     *
  *   it under the terms of the GNU General Public License as published    *
@@ -21,7 +21,7 @@
  *                                                                        *
  **************************************************************************/
 
-#include "proto.h"
+#include "prototypes.h"
 
 #ifdef ENABLE_NANORC
 
@@ -32,10 +32,10 @@
 #include <unistd.h>
 
 #ifndef RCFILE_NAME
-#define HOME_RC_NAME ".nanorc"
-#define RCFILE_NAME "nanorc"
+#define HOME_RC_NAME  ".nanorc"
+#define RCFILE_NAME  "nanorc"
 #else
-#define HOME_RC_NAME RCFILE_NAME
+#define HOME_RC_NAME  RCFILE_NAME
 #endif
 
 static const rcoption rcopts[] = {
@@ -46,6 +46,7 @@ static const rcoption rcopts[] = {
 #ifdef ENABLE_WRAPPING
 	{"breaklonglines", BREAK_LONG_LINES},
 #endif
+	{"casesensitive", CASE_SENSITIVE},
 	{"constantshow", CONSTANT_SHOW},
 	{"emptyline", EMPTY_LINE},
 #ifdef ENABLED_WRAPORJUSTIFY
@@ -86,23 +87,26 @@ static const rcoption rcopts[] = {
 	{"rawsequences", RAW_SEQUENCES},
 	{"rebinddelete", REBIND_DELETE},
 	{"regexp", USE_REGEXP},
+	{"saveonexit", SAVE_ON_EXIT},
 #ifdef ENABLE_SPELLER
 	{"speller", 0},
 #endif
-	{"suspend", SUSPEND},
+	{"suspend", SUSPENDABLE},  /* Deprecated; remove in 2022. */
+	{"suspendable", SUSPENDABLE},
 	{"tabsize", 0},
-	{"tempfile", TEMP_FILE},
+	{"tempfile", SAVE_ON_EXIT},  /* Deprecated; remove in 2022. */
 	{"view", VIEW_MODE},
 #ifndef NANO_TINY
 	{"afterends", AFTER_ENDS},
 	{"allow_insecure_backup", INSECURE_BACKUP},
 	{"atblanks", AT_BLANKS},
 	{"autoindent", AUTOINDENT},
-	{"backup", BACKUP_FILE},
+	{"backup", MAKE_BACKUP},
 	{"backupdir", 0},
-	{"casesensitive", CASE_SENSITIVE},
+	{"bookstyle", BOOKSTYLE},
 	{"cutfromcursor", CUT_FROM_CURSOR},
 	{"guidestripe", 0},
+	{"indicator", INDICATOR},
 	{"locking", LOCKING},
 	{"matchbrackets", 0},
 	{"noconvert", NO_CONVERT},
@@ -147,15 +151,15 @@ static colortype *lastcolor = NULL;
 		/* The end of the color list for the current syntax. */
 #endif
 
-#define NUMBER_OF_MENUS  16
+#define NUMBER_OF_MENUS  17  /* Remove the deprecated 'extcmd' in 2022. */
 char *menunames[NUMBER_OF_MENUS] = { "main", "search", "replace", "replacewith",
 									"yesno", "gotoline", "writeout", "insert",
-									"extcmd", "help", "spell", "linter",
+									"execute", "extcmd", "help", "spell", "linter",
 									"browser", "whereisfile", "gotodir",
 									"all" };
 int menusymbols[NUMBER_OF_MENUS] = { MMAIN, MWHEREIS, MREPLACE, MREPLACEWITH,
 									MYESNO, MGOTOLINE, MWRITEFILE, MINSERTFILE,
-									MEXTCMD, MHELP, MSPELL, MLINTER,
+									MEXECUTE, MEXECUTE, MHELP, MSPELL, MLINTER,
 									MBROWSER, MWHEREISFILE, MGOTODIR,
 									MMOST|MBROWSER|MHELP|MYESNO };
 #endif /* ENABLE_NANORC */
@@ -253,6 +257,8 @@ keystruct *strtosc(const char *input)
 	else if (!strcmp(input, "paste"))
 		s->func = paste_text;
 #ifndef NANO_TINY
+	else if (!strcmp(input, "execute"))
+		s->func = do_execute;
 	else if (!strcmp(input, "cutrestoffile"))
 		s->func = cut_till_eof;
 	else if (!strcmp(input, "copy"))
@@ -264,19 +270,18 @@ keystruct *strtosc(const char *input)
 #endif
 #ifdef ENABLE_SPELLER
 	else if (!strcmp(input, "tospell") ||
-			 !strcmp(input, "speller"))
+	         !strcmp(input, "speller"))
 		s->func = do_spell;
 #endif
 #ifdef ENABLE_COLOR
 	else if (!strcmp(input, "linter"))
 		s->func = do_linter;
-#ifdef ENABLE_SPELLER
 	else if (!strcmp(input, "formatter"))
 		s->func = do_formatter;
 #endif
-#endif
-	else if (!strcmp(input, "curpos"))
-		s->func = do_cursorpos_void;
+	else if (!strcmp(input, "location") ||
+	         !strcmp(input, "curpos"))  /* Deprecated; remove in 2022. */
+		s->func = report_cursor_position;
 	else if (!strcmp(input, "gotoline"))
 		s->func = do_gotolinecolumn_void;
 #ifdef ENABLE_JUSTIFY
@@ -285,9 +290,9 @@ keystruct *strtosc(const char *input)
 	else if (!strcmp(input, "fulljustify"))
 		s->func = do_full_justify;
 	else if (!strcmp(input, "beginpara"))
-		s->func = do_para_begin_void;
+		s->func = to_para_begin;
 	else if (!strcmp(input, "endpara"))
-		s->func = do_para_end_void;
+		s->func = to_para_end;
 #endif
 #ifdef ENABLE_COMMENT
 	else if (!strcmp(input, "comment"))
@@ -303,10 +308,10 @@ keystruct *strtosc(const char *input)
 	else if (!strcmp(input, "unindent"))
 		s->func = do_unindent;
 	else if (!strcmp(input, "chopwordleft") ||
-			 !strcmp(input, "cutwordleft"))  /* Deprecated; remove in 2021. */
+	         !strcmp(input, "cutwordleft"))  /* Deprecated; remove in 2021. */
 		s->func = chop_previous_word;
 	else if (!strcmp(input, "chopwordright") ||
-			 !strcmp(input, "cutwordright"))  /* Deprecated; remove in 2021. */
+	         !strcmp(input, "cutwordright"))  /* Deprecated; remove in 2021. */
 		s->func = chop_next_word;
 	else if (!strcmp(input, "findbracket"))
 		s->func = do_find_bracket;
@@ -316,46 +321,54 @@ keystruct *strtosc(const char *input)
 		s->func = record_macro;
 	else if (!strcmp(input, "runmacro"))
 		s->func = run_macro;
+	else if (!strcmp(input, "anchor"))
+		s->func = put_or_lift_anchor;
+	else if (!strcmp(input, "prevanchor"))
+		s->func = to_prev_anchor;
+	else if (!strcmp(input, "nextanchor"))
+		s->func = to_next_anchor;
 	else if (!strcmp(input, "undo"))
 		s->func = do_undo;
 	else if (!strcmp(input, "redo"))
 		s->func = do_redo;
 #endif
 	else if (!strcmp(input, "left") ||
-			 !strcmp(input, "back"))
+	         !strcmp(input, "back"))
 		s->func = do_left;
 	else if (!strcmp(input, "right") ||
-			 !strcmp(input, "forward"))
+	         !strcmp(input, "forward"))
 		s->func = do_right;
 	else if (!strcmp(input, "up") ||
-			 !strcmp(input, "prevline"))
+	         !strcmp(input, "prevline"))
 		s->func = do_up;
 	else if (!strcmp(input, "down") ||
-			 !strcmp(input, "nextline"))
+	         !strcmp(input, "nextline"))
 		s->func = do_down;
 #if !defined(NANO_TINY) || defined(ENABLE_HELP)
 	else if (!strcmp(input, "scrollup"))
 		s->func = do_scroll_up;
 	else if (!strcmp(input, "scrolldown"))
 		s->func = do_scroll_down;
+	else if (!strcmp(input, "center"))
+		s->func = do_center;
 #endif
 	else if (!strcmp(input, "prevword"))
-		s->func = do_prev_word_void;
+		s->func = to_prev_word;
 	else if (!strcmp(input, "nextword"))
-		s->func = do_next_word_void;
+		s->func = to_next_word;
 	else if (!strcmp(input, "home"))
 		s->func = do_home;
 	else if (!strcmp(input, "end"))
 		s->func = do_end;
 	else if (!strcmp(input, "prevblock"))
-		s->func = do_prev_block;
+		s->func = to_prev_block;
 	else if (!strcmp(input, "nextblock"))
-		s->func = do_next_block;
+		s->func = to_next_block;
 	else if (!strcmp(input, "pageup") ||
-			 !strcmp(input, "prevpage"))
+	         !strcmp(input, "prevpage"))
 		s->func = do_page_up;
 	else if (!strcmp(input, "pagedown") ||
-			 !strcmp(input, "nextpage"))
+	         !strcmp(input, "nextpage"))
 		s->func = do_page_down;
 	else if (!strcmp(input, "firstline"))
 		s->func = to_first_line;
@@ -378,7 +391,7 @@ keystruct *strtosc(const char *input)
 	else if (!strcmp(input, "backspace"))
 		s->func = do_backspace;
 	else if (!strcmp(input, "refresh"))
-		s->func = total_refresh;
+		s->func = full_refresh;
 	else if (!strcmp(input, "suspend"))
 		s->func = do_suspend_void;
 	else if (!strcmp(input, "casesens"))
@@ -421,7 +434,7 @@ keystruct *strtosc(const char *input)
 #endif
 #ifdef ENABLE_BROWSER
 	else if (!strcmp(input, "tofiles") ||
-			 !strcmp(input, "browser"))
+	         !strcmp(input, "browser"))
 		s->func = to_files;
 	else if (!strcmp(input, "gotodir"))
 		s->func = goto_dir;
@@ -447,7 +460,7 @@ keystruct *strtosc(const char *input)
 			s->toggle = WHITESPACE_DISPLAY;
 #ifdef ENABLE_COLOR
 		else if (!strcmp(input, "nosyntax"))
-			s->toggle = NO_COLOR_SYNTAX;
+			s->toggle = NO_SYNTAX;
 #endif
 		else if (!strcmp(input, "smarthome"))
 			s->toggle = SMART_HOME;
@@ -465,8 +478,9 @@ keystruct *strtosc(const char *input)
 		else if (!strcmp(input, "mouse"))
 			s->toggle = USE_MOUSE;
 #endif
-		else if (!strcmp(input, "suspendenable"))
-			s->toggle = SUSPEND;
+		else if (!strcmp(input, "suspendable") ||
+		         !strcmp(input, "suspendenable"))  /* Deprecated; remove in 2022. */
+			s->toggle = SUSPENDABLE;
 		else
 #endif /* !NANO_TINY */
 		{
@@ -553,14 +567,21 @@ char *parse_argument(char *ptr)
 }
 
 #ifdef ENABLE_COLOR
-/* Pass over the current regex string in the line starting at ptr,
- * null-terminate it, and return a pointer to the /next/ word. */
+/* Advance over one regular expression in the line starting at ptr,
+ * null-terminate it, and return a pointer to the succeeding text. */
 char *parse_next_regex(char *ptr)
 {
-	/* Continue until the end of line, or until a " followed by a
-	 * blank character or the end of line. */
+	char *starting_point = ptr;
+
+	if (*(ptr - 1) != '"') {
+		jot_error(N_("Regex strings must begin and end with a \" character"));
+		return NULL;
+	}
+
+	/* Continue until the end of the line, or until a double quote followed
+	 * by end-of-line or a blank. */
 	while (*ptr != '\0' && (*ptr != '"' ||
-				(*(ptr + 1) != '\0' && !isblank((unsigned char)ptr[1]))))
+						(ptr[1] != '\0' && !isblank((unsigned char)ptr[1]))))
 		ptr++;
 
 	if (*ptr == '\0') {
@@ -568,7 +589,12 @@ char *parse_next_regex(char *ptr)
 		return NULL;
 	}
 
-	/* Null-terminate and advance ptr. */
+	if (ptr == starting_point) {
+		jot_error(N_("Empty regex string"));
+		return NULL;
+	}
+
+	/* Null-terminate the regex and skip until the next non-blank. */
 	*ptr++ = '\0';
 
 	while (isblank((unsigned char)*ptr))
@@ -694,7 +720,7 @@ bool is_universal(void (*func)(void))
 	return (func == do_left || func == do_right ||
 		func == do_home || func == do_end ||
 #ifndef NANO_TINY
-		func == do_prev_word_void || func == do_next_word_void ||
+		func == to_prev_word || func == to_next_word ||
 #endif
 		func == do_delete || func == do_backspace ||
 		func == cut_text || func == paste_text ||
@@ -794,13 +820,17 @@ void parse_binding(char *ptr, bool dobind)
 		goto free_things;
 
 	/* Limit the given menu to those where the function exists;
-	 * first handle three special cases, then the general case. */
+	 * first handle five special cases, then the general case. */
 	if (is_universal(newsc->func))
 		menu &= MMOST|MBROWSER;
 #ifndef NANO_TINY
+	else if (newsc->func == do_toggle_void && newsc->toggle == NO_HELP)
+		menu &= (MMOST|MBROWSER|MYESNO) & ~MFINDINHELP;
 	else if (newsc->func == do_toggle_void)
 		menu &= MMAIN;
 #endif
+	else if (newsc->func == full_refresh)
+		menu &= MMOST|MBROWSER|MHELP|MYESNO;
 	else if (newsc->func == (functionptrtype)implant)
 		menu &= MMOST|MBROWSER|MHELP;
 	else {
@@ -958,15 +988,26 @@ void parse_includes(char *ptr)
 	free(expanded);
 }
 
-/* Return the short value corresponding to the color named in colorname,
- * and set bright to TRUE if that color is bright. */
-short color_to_short(const char *colorname, bool *bright)
+const char hues[9][7] = { "pink", "purple", "mauve", "lagoon", "mint",
+						  "lime", "peach", "orange", "latte" };
+short indices[9] = { 204, 163, 134, 38, 48, 148, 215, 208, 137 };
+
+/* Return the short value corresponding to the given color name, and set
+ * vivid to TRUE for a lighter color, and thick for a heavier typeface. */
+short color_to_short(const char *colorname, bool *vivid, bool *thick)
 {
-	if (strncmp(colorname, "bright", 6) == 0) {
-		*bright = TRUE;
+	if (strncmp(colorname, "bright", 6) == 0) {  /* Deprecated; remove in 2023. */
+		*vivid = TRUE;
+		*thick = TRUE;
 		colorname += 6;
-	} else
-		*bright = FALSE;
+	} else if (strncmp(colorname, "light", 5) == 0) {
+		*vivid = TRUE;
+		*thick = FALSE;
+		colorname += 5;
+	} else {
+		*vivid = FALSE;
+		*thick = FALSE;
+	}
 
 	if (strcmp(colorname, "green") == 0)
 		return COLOR_GREEN;
@@ -985,7 +1026,18 @@ short color_to_short(const char *colorname, bool *bright)
 	else if (strcmp(colorname, "black") == 0)
 		return COLOR_BLACK;
 	else if (strcmp(colorname, "normal") == 0)
-		return USE_THE_DEFAULT;
+		return THE_DEFAULT;
+	else
+		for (int index = 0; index < 9; index++)
+			if (strcmp(colorname, hues[index]) == 0) {
+				if (*vivid) {
+					jot_error(N_("Color '%s' takes no prefix"), colorname);
+					return BAD_COLOR;
+				} else if (COLORS < 255)
+					return THE_DEFAULT;
+				else
+					return indices[index];
+			}
 
 	jot_error(N_("Color \"%s\" not understood"), colorname);
 	return BAD_COLOR;
@@ -993,55 +1045,79 @@ short color_to_short(const char *colorname, bool *bright)
 
 /* Parse the color name (or pair of color names) in the given string.
  * Return FALSE when any color name is invalid; otherwise return TRUE. */
-bool parse_color_names(char *combostr, short *fg, short *bg, int *attributes)
+bool parse_combination(char *combostr, short *fg, short *bg, int *attributes)
 {
-	char *comma = strchr(combostr, ',');
-	bool bright;
+	bool vivid, thick;
+	char *comma;
 
 	*attributes = A_NORMAL;
 
-	if (comma != NULL) {
-		*bg = color_to_short(comma + 1, &bright);
-		if (bright) {
-			jot_error(N_("A background color cannot be bright"));
+	if (strncmp(combostr, "bold", 4) == 0) {
+		*attributes |= A_BOLD;
+		if (combostr[4] != ',') {
+			jot_error(N_("An attribute requires a subsequent comma"));
 			return FALSE;
 		}
-		if (*bg == BAD_COLOR)
-			return FALSE;
-		*comma = '\0';
-	} else
-		*bg = USE_THE_DEFAULT;
+		combostr += 5;
+	}
 
-	if (comma != combostr) {
-		*fg = color_to_short(combostr, &bright);
+	if (strncmp(combostr, "italic", 6) == 0) {
+#ifdef A_ITALIC
+		*attributes |= A_ITALIC;
+#endif
+		if (combostr[6] != ',') {
+			jot_error(N_("An attribute requires a subsequent comma"));
+			return FALSE;
+		}
+		combostr += 7;
+	}
+
+	comma = strchr(combostr, ',');
+
+	if (comma)
+		*comma = '\0';
+
+	if (!comma || comma > combostr) {
+		*fg = color_to_short(combostr, &vivid, &thick);
 		if (*fg == BAD_COLOR)
 			return FALSE;
-
-		if (bright)
-			*attributes = A_BOLD;
+		if (vivid && !thick && COLORS > 8)
+			*fg += 8;
+		else if (vivid)
+			*attributes |= A_BOLD;
 	} else
-		*fg = USE_THE_DEFAULT;
+		*fg = THE_DEFAULT;
+
+	if (comma) {
+		*bg = color_to_short(comma + 1, &vivid, &thick);
+		if (*bg == BAD_COLOR)
+			return FALSE;
+		if (vivid && COLORS > 8)
+			*bg += 8;
+	} else
+		*bg = THE_DEFAULT;
 
 	return TRUE;
 }
 
-/* Parse the color string in the line at ptr, and add it to the current
- * file's associated colors.  rex_flags are the regex compilation flags
- * to use, excluding or including REG_ICASE for case (in)sensitivity. */
-void parse_colors(char *ptr, int rex_flags)
+/* Parse the color specification that starts at ptr, and then the one or more
+ * regexes that follow it.  For each valid regex (or start=/end= regex pair),
+ * add a rule to the current syntax. */
+void parse_rule(char *ptr, int rex_flags)
 {
+	char *names, *regexstring;
 	short fg, bg;
 	int attributes;
-	char *item;
 
 	if (*ptr == '\0') {
 		jot_error(N_("Missing color name"));
 		return;
 	}
 
-	item = ptr;
+	names = ptr;
 	ptr = parse_next_word(ptr);
-	if (!parse_color_names(item, &fg, &bg, &attributes))
+
+	if (!parse_combination(names, &fg, &bg, &attributes))
 		return;
 
 	if (*ptr == '\0') {
@@ -1049,93 +1125,68 @@ void parse_colors(char *ptr, int rex_flags)
 		return;
 	}
 
-	/* Now for the fun part.  Start adding regexes to individual strings
-	 * in the colorstrings array, woo! */
-	while (ptr != NULL && *ptr != '\0') {
+	while (*ptr != '\0') {
+		regex_t *start_rgx = NULL, *end_rgx = NULL;
+			/* Intermediate storage for compiled regular expressions. */
 		colortype *newcolor = NULL;
-			/* The container for a color plus its regexes. */
-		bool goodstart;
-			/* Whether the start expression was valid. */
+			/* Container for compiled regex (pair) and the color it paints. */
 		bool expectend = FALSE;
-			/* Whether to expect an end= line. */
+			/* Whether it is a start=/end= regex pair. */
 
 		if (strncmp(ptr, "start=", 6) == 0) {
 			ptr += 6;
 			expectend = TRUE;
 		}
 
-		if (*ptr != '"') {
-			jot_error(N_("Regex strings must begin and end with a \" character"));
-			ptr = parse_next_regex(ptr);
-			continue;
-		}
-
-		item = ++ptr;
+		regexstring = ++ptr;
 		ptr = parse_next_regex(ptr);
-		if (ptr == NULL)
-			break;
 
-		if (*item == '\0') {
-			jot_error(N_("Empty regex string"));
-			goodstart = FALSE;
-		} else {
-			newcolor = (colortype *)nmalloc(sizeof(colortype));
-			goodstart = compile(item, rex_flags, &newcolor->start);
-		}
-
-		/* If the start regex is valid, fill in the rest of the data, and
-		 * hook the new color struct in at the tail of the linked list. */
-		if (goodstart) {
-			newcolor->fg = fg;
-			newcolor->bg = bg;
-			newcolor->attributes = attributes;
-
-			newcolor->end = NULL;
-			newcolor->next = NULL;
-
-			if (lastcolor == NULL)
-				live_syntax->color = newcolor;
-			else
-				lastcolor->next = newcolor;
-
-			lastcolor = newcolor;
-		} else
-			free(newcolor);
-
-		if (!expectend)
-			continue;
-
-		if (ptr == NULL || strncmp(ptr, "end=", 4) != 0) {
-			jot_error(N_("\"start=\" requires a corresponding \"end=\""));
+		/* When there is no regex, or it is invalid, skip this line. */
+		if (ptr == NULL || !compile(regexstring, rex_flags, &start_rgx))
 			return;
+
+		if (expectend) {
+			if (strncmp(ptr, "end=", 4) != 0) {
+				jot_error(N_("\"start=\" requires a corresponding \"end=\""));
+				regfree(start_rgx);
+				free(start_rgx);
+				return;
+			}
+
+			regexstring = ptr + 5;
+			ptr = parse_next_regex(ptr + 5);
+
+			/* When there is no valid end= regex, abandon the rule. */
+			if (ptr == NULL || !compile(regexstring, rex_flags, &end_rgx)) {
+				regfree(start_rgx);
+				free(start_rgx);
+				return;
+			}
 		}
 
-		ptr += 4;
-		if (*ptr != '"') {
-			jot_error(N_("Regex strings must begin and end with a \" character"));
-			continue;
+		/* Allocate a rule, fill in the data, and link it into the list. */
+		newcolor = (colortype *)nmalloc(sizeof(colortype));
+
+		newcolor->start = start_rgx;
+		newcolor->end = end_rgx;
+
+		newcolor->fg = fg;
+		newcolor->bg = bg;
+		newcolor->attributes = attributes;
+
+		if (lastcolor == NULL)
+			live_syntax->color = newcolor;
+		else
+			lastcolor->next = newcolor;
+
+		newcolor->next = NULL;
+		lastcolor = newcolor;
+
+		/* For a multiline rule, give it a number and increase the count. */
+		if (expectend) {
+			newcolor->id = live_syntax->nmultis;
+			live_syntax->nmultis++;
 		}
-
-		item = ++ptr;
-		ptr = parse_next_regex(ptr);
-		if (ptr == NULL)
-			break;
-
-		if (*item == '\0') {
-			jot_error(N_("Empty regex string"));
-			continue;
-		}
-
-		/* If the start regex was invalid, the end regex cannot be saved. */
-		if (!goodstart)
-			continue;
-
-		/* Save the compiled ending regex (when it's valid). */
-		compile(item, rex_flags, &newcolor->end);
-
-		/* Lame way to skip another static counter. */
-		newcolor->id = live_syntax->nmultis;
-		live_syntax->nmultis++;
 	}
 }
 
@@ -1144,7 +1195,7 @@ colortype *parse_interface_color(char *combostr)
 {
 	colortype *trio = nmalloc(sizeof(colortype));
 
-	if (parse_color_names(combostr, &trio->fg, &trio->bg, &trio->attributes)) {
+	if (parse_combination(combostr, &trio->fg, &trio->bg, &trio->attributes)) {
 		free(combostr);
 		return trio;
 	} else {
@@ -1158,7 +1209,8 @@ colortype *parse_interface_color(char *combostr)
  * by ptr, and store them quoteless in the passed storage place. */
 void grab_and_store(const char *kind, char *ptr, regexlisttype **storage)
 {
-	regexlisttype *lastthing;
+	regexlisttype *lastthing, *newthing;
+	const char *regexstring;
 
 	if (!opensyntax) {
 		jot_error(N_("A '%s' command requires a preceding 'syntax' command"), kind);
@@ -1184,16 +1236,9 @@ void grab_and_store(const char *kind, char *ptr, regexlisttype **storage)
 
 	/* Now gather any valid regexes and add them to the linked list. */
 	while (*ptr != '\0') {
-		const char *regexstring;
-		regexlisttype *newthing;
-
-		if (*ptr != '"') {
-			jot_error(N_("Regex strings must begin and end with a \" character"));
-			return;
-		}
-
 		regexstring = ++ptr;
 		ptr = parse_next_regex(ptr);
+
 		if (ptr == NULL)
 			return;
 
@@ -1241,21 +1286,19 @@ void pick_up_name(const char *kind, char *ptr, char **storage)
 	*storage = mallocstrcpy(*storage, ptr);
 }
 
-/* Handle the four syntax-only commands. */
+/* Handle the six syntax-only commands. */
 bool parse_syntax_commands(char *keyword, char *ptr)
 {
 	if (strcmp(keyword, "color") == 0)
-		parse_colors(ptr, NANO_REG_EXTENDED);
+		parse_rule(ptr, NANO_REG_EXTENDED);
 	else if (strcmp(keyword, "icolor") == 0)
-		parse_colors(ptr, NANO_REG_EXTENDED | REG_ICASE);
+		parse_rule(ptr, NANO_REG_EXTENDED | REG_ICASE);
 	else if (strcmp(keyword, "comment") == 0) {
 #ifdef ENABLE_COMMENT
 		pick_up_name("comment", ptr, &live_syntax->comment);
 #endif
 	} else if (strcmp(keyword, "tabgives") == 0) {
-#ifdef ENABLE_COLOR
 		pick_up_name("tabgives", ptr, &live_syntax->tab);
-#endif
 	} else if (strcmp(keyword, "linter") == 0)
 		pick_up_name("linter", ptr, &live_syntax->linter);
 	else if (strcmp(keyword, "formatter") == 0)
@@ -1278,16 +1321,13 @@ static void check_vitals_mapped(void)
 	for (int v = 0; v < VITALS; v++) {
 		for (funcstruct *f = allfuncs; f != NULL; f = f->next) {
 			if (f->func == vitals[v] && f->menus & inmenus[v]) {
-				const keystruct *s = first_sc_for(inmenus[v], f->func);
-				if (!s) {
-					fprintf(stderr, _("No key is bound to function '%s' in "
-										"menu '%s'.  Exiting.\n"), f->desc,
-										menu_to_name(inmenus[v]));
-					fprintf(stderr, _("If needed, use nano with the -I option "
-										"to adjust your nanorc settings.\n"));
-					exit(1);
-				}
-				break;
+				if (first_sc_for(inmenus[v], f->func) == NULL) {
+					jot_error(N_("No key is bound to function '%s' in menu '%s'. "
+								" Exiting.\n"), f->desc, menu_to_name(inmenus[v]));
+					die(_("If needed, use nano with the -I option "
+								"to adjust your nanorc settings.\n"));
+				} else
+					break;
 			}
 		}
 	}
@@ -1299,10 +1339,10 @@ static void check_vitals_mapped(void)
 void parse_rcfile(FILE *rcstream, bool just_syntax, bool intros_only)
 {
 	char *buffer = NULL;
-	ssize_t len;
 	size_t size = 0;
+	ssize_t length;
 
-	while ((len = getline(&buffer, &size, rcstream)) > 0) {
+	while ((length = getline(&buffer, &size, rcstream)) > 0) {
 		char *ptr, *keyword, *option, *argument;
 #ifdef ENABLE_COLOR
 		bool drop_open = FALSE;
@@ -1317,9 +1357,11 @@ void parse_rcfile(FILE *rcstream, bool just_syntax, bool intros_only)
 		if (just_syntax && !intros_only && lineno <= live_syntax->lineno)
 			continue;
 #endif
-		/* Strip the terminating newline, if any. */
-		if (buffer[len - 1] == '\n')
-			buffer[len - 1] = '\0';
+		/* Strip the terminating newline and possibly a carriage return. */
+		if (buffer[length - 1] == '\n')
+			buffer[--length] = '\0';
+		if (length > 0 && buffer[length - 1] == '\r')
+			buffer[--length] = '\0';
 
 		ptr = buffer;
 		while (isblank((unsigned char)*ptr))
@@ -1338,17 +1380,17 @@ void parse_rcfile(FILE *rcstream, bool just_syntax, bool intros_only)
 		if (!just_syntax && strcmp(keyword, "extendsyntax") == 0) {
 			augmentstruct *newitem, *extra;
 			char *syntaxname = ptr;
-			syntaxtype *sint;
+			syntaxtype *sntx;
 
 			check_for_nonempty_syntax();
 
 			ptr = parse_next_word(ptr);
 
-			for (sint = syntaxes; sint != NULL; sint = sint->next)
-				if (!strcmp(sint->name, syntaxname))
+			for (sntx = syntaxes; sntx != NULL; sntx = sntx->next)
+				if (!strcmp(sntx->name, syntaxname))
 					break;
 
-			if (sint == NULL) {
+			if (sntx == NULL) {
 				jot_error(N_("Could not find syntax \"%s\" to extend"), syntaxname);
 				continue;
 			}
@@ -1361,7 +1403,7 @@ void parse_rcfile(FILE *rcstream, bool just_syntax, bool intros_only)
 			 * other commands are stored for possible later processing. */
 			if (strcmp(keyword, "header") == 0 || strcmp(keyword, "magic") == 0) {
 				free(argument);
-				live_syntax = sint;
+				live_syntax = sntx;
 				opensyntax = TRUE;
 				drop_open = TRUE;
 			} else {
@@ -1372,13 +1414,13 @@ void parse_rcfile(FILE *rcstream, bool just_syntax, bool intros_only)
 				newitem->data = argument;
 				newitem->next = NULL;
 
-				if (sint->augmentations != NULL) {
-					extra = sint->augmentations;
+				if (sntx->augmentations != NULL) {
+					extra = sntx->augmentations;
 					while (extra->next != NULL)
 						extra = extra->next;
 					extra->next = newitem;
 				} else
-					sint->augmentations = newitem;
+					sntx->augmentations = newitem;
 
 				continue;
 			}
@@ -1419,7 +1461,7 @@ void parse_rcfile(FILE *rcstream, bool just_syntax, bool intros_only)
 			if (!opensyntax)
 				jot_error(N_("A '%s' command requires a preceding "
 									"'syntax' command"), keyword);
-			if (strcasestr("icolor", keyword))
+			if (strstr("icolor", keyword))
 				seen_color_command = TRUE;
 			continue;
 		} else if (parse_syntax_commands(keyword, ptr))
